==> Rredland_1.4.0.tar.gz: Rredland/src/Rredl.c <==
#include <stdio.h>
#include <redland.h>

#include <R.h>
#include <Rdefines.h>
#include <Rinternals.h>

/* note -- when adding a thing, increment the NUM_REDL_THINGS, and
   add a CHECK macro below */

#define NUM_REDL_THINGS 11
#define IS_REDL_SUBJECT 0
#define IS_REDL_PREDICATE 1
#define IS_REDL_OBJECT 2

enum redl_things {
	redl_world,
	redl_parser,
	redl_model,
	redl_storage,
	redl_stream,
	redl_subject,
	redl_predicate,
	redl_iterator,
	redl_statement,
	redl_node,
	redl_uri
};

static SEXP redl_type_[ NUM_REDL_THINGS ] ;

#define CHECK_REDL_REF(s,THING,CTYPE) do { \
	if (TYPEOF(s) != EXTPTRSXP || \
		R_ExternalPtrTag(s) != redl_type_[ THING ] ) \
		error("bad redland"  #CTYPE "reference"); \
	if ( (CTYPE *)R_ExternalPtrAddr(s) == (CTYPE *)NULL ) \
		error("CTYPE pointer is null"); \
	} while (0) ;

#define CHECK_REDL_WORLD(s) CHECK_REDL_REF(s, redl_world, librdf_world)
#define CHECK_REDL_PARSER(s) CHECK_REDL_REF(s, redl_parser, librdf_parser)
#define CHECK_REDL_MODEL(s) CHECK_REDL_REF(s, redl_model, librdf_model)
#define CHECK_REDL_STORAGE(s) CHECK_REDL_REF(s, redl_storage, librdf_storage)
#define CHECK_REDL_STREAM(s) CHECK_REDL_REF(s, redl_stream, librdf_stream)
#define CHECK_REDL_SUBJECT(s) CHECK_REDL_REF(s, redl_subject, librdf_subject)
#define CHECK_REDL_PREDICATE(s) CHECK_REDL_REF(s, redl_predicate, librdf_predicate)
#define CHECK_REDL_ITERATOR(s) CHECK_REDL_REF(s, redl_iterator, librdf_iterator)
#define CHECK_REDL_STATEMENT(s) CHECK_REDL_REF(s, redl_statement, librdf_statement)
#define CHECK_REDL_NODE(s) CHECK_REDL_REF(s, redl_node, librdf_node)
#define CHECK_REDL_URI(s) CHECK_REDL_REF(s, redl_uri, librdf_uri)
	

/*

openRedlWorld <- function() {
	.Call("get_open_redl_world", PACKAGE="Rredl" )
}
getRedlURI <- function(worldRef, uri) {
	if (!is(uri,"character") | length(uri)>1) stop("uri must be character string")
	.Call("set_redl_world_uri", worldRef, uri, PACKAGE="Rredl" )
}
getRedlStorageInCore <- function(worldRef) {
	.Call("get_storage_incore", worldRef, PACKAGE="Rredl" )
}
getRedlModel <- function(worldRef,storageRef) {
	.Call("get_redl_model", worldRef, storageRef, PACKAGE="Rredl" )
}
getRedlDefParser <- function(worldRef) {
	.Call("get_redl_default_parser", worldRef, PACKAGE="Rredl" )
}
 
*/
SEXP get_open_redl_world() {
	/* return an R external pointer to an open redl world */

	librdf_world* world;
	SEXP retptr;

	world=librdf_new_world();
	if (!world) error("could not create new world.");
	librdf_world_open(world);
	if (!world) error("could not open new world.");

	retptr = R_MakeExternalPtr( world, redl_type_[ redl_world ], R_NilValue );
	return(retptr);
}
SEXP free_redl_world(SEXP inworld) {
	librdf_world* world;
	CHECK_REDL_WORLD(inworld); 
	world = (librdf_world *) R_ExternalPtrAddr( inworld );
  	librdf_free_world(world);
	return(R_NilValue);
}
SEXP free_redl_storage(SEXP instorage) {
	librdf_storage* storage;
	CHECK_REDL_STORAGE(instorage);
	storage = (librdf_storage *) R_ExternalPtrAddr( instorage );
  	librdf_free_storage(storage);
	return(R_NilValue);
}
SEXP free_redl_model(SEXP inmodel) {
	librdf_model* model;
	CHECK_REDL_STORAGE(inmodel);
	model = (librdf_model *) R_ExternalPtrAddr( inmodel );
  	librdf_free_model(model);
	return(R_NilValue);
}
void get_and_free_redl_world() {
	librdf_world* world;
	world=librdf_new_world();
	librdf_world_open(world);
  	librdf_free_world(world);
}

SEXP get_redl_world_uri(SEXP inworld, SEXP inuri) {
	/* establish the source URI of RDF for a world */
	librdf_uri* uri;
	librdf_world* world;
	unsigned char* uristring;
	SEXP retptr;

	CHECK_REDL_WORLD(inworld);
	world = (librdf_world *) R_ExternalPtrAddr( inworld );

	uristring = CHAR(STRING_ELT(inuri,0));
	uri = librdf_new_uri( world, uristring );
	if(!uri) error("Failed to create URI");
	retptr = R_MakeExternalPtr( uri, redl_type_[ redl_uri ], R_NilValue );
	return(retptr);
}

SEXP new_storage_incore(SEXP inworld) {
	librdf_storage* storage;
	librdf_world* world;
	SEXP retptr;

	CHECK_REDL_WORLD(inworld);
	world = (librdf_world *) R_ExternalPtrAddr( inworld );
  	storage=librdf_new_storage(world, "memory", "test", NULL);
	if(storage == NULL) error("failed to create new storage");
	retptr = R_MakeExternalPtr( storage, redl_type_[ redl_storage ], R_NilValue );
	return(retptr);
}

SEXP call_new_storage(SEXP inworld, SEXP type,
		SEXP name, SEXP rshash) {
	librdf_storage* storage;
	librdf_world* world;
	SEXP retptr;
	CHECK_REDL_WORLD(inworld);
	world = (librdf_world *) R_ExternalPtrAddr( inworld );
 	if (strlen(CHAR(STRING_ELT(rshash,0))) == 0)
  		storage=librdf_new_storage(world, CHAR(STRING_ELT(type,0)), 
			CHAR(STRING_ELT(name,0)),
			NULL);
	else
  		storage=librdf_new_storage(world, CHAR(STRING_ELT(type,0)), 
			CHAR(STRING_ELT(name,0)),
			CHAR(STRING_ELT(rshash,0)));
	if(storage == NULL) error("failed to create new storage");
	retptr = R_MakeExternalPtr( storage, redl_type_[ redl_storage ], R_NilValue );
	return(retptr);
}

SEXP new_storage_bdb(SEXP inworld, SEXP stoname) {
	librdf_storage* storage;
	librdf_world* world;
	SEXP retptr;

	CHECK_REDL_WORLD(inworld);
	world = (librdf_world *) R_ExternalPtrAddr( inworld );
  	storage=librdf_new_storage(world, "hashes", CHAR(STRING_ELT(stoname,0)), "hash-type='bdb',dir='.'");
	if(storage == NULL) error("failed to create new storage");
	retptr = R_MakeExternalPtr( storage, redl_type_[ redl_storage ], R_NilValue );
	return(retptr);
}

SEXP revive_storage_testbdb(SEXP inworld, SEXP stoname) {
	librdf_storage* storage;
	librdf_world* world;
	SEXP retptr;

	CHECK_REDL_WORLD(inworld);
	world = (librdf_world *) R_ExternalPtrAddr( inworld );
  	storage=librdf_new_storage(world, "hashes", CHAR(STRING_ELT(stoname,0)), "hash-type='bdb',dir='.'");
	if(storage == NULL) error("failed to revive existing storage");
	retptr = R_MakeExternalPtr( storage, redl_type_[ redl_storage ], R_NilValue );
	return(retptr);
}

SEXP get_redl_model(SEXP inworld, SEXP instorage) {
	librdf_storage* storage;
	librdf_world* world;
	librdf_model* model;
	SEXP retptr;

	CHECK_REDL_WORLD(inworld);
	world = (librdf_world *) R_ExternalPtrAddr( inworld );
	CHECK_REDL_STORAGE(instorage);
	storage = (librdf_storage *) R_ExternalPtrAddr( instorage );

  	model=librdf_new_model(world, storage, NULL);
	if(!model) error("failed to create new model");
	retptr = R_MakeExternalPtr( model, redl_type_[ redl_model ], R_NilValue );
	return(retptr);
}

SEXP get_redl_default_parser(SEXP inworld) {
	librdf_parser* parser;
	librdf_world* world;
	SEXP retptr;

	CHECK_REDL_WORLD(inworld);
	world = (librdf_world *) R_ExternalPtrAddr( inworld );

	parser=librdf_new_parser(world, NULL, NULL, NULL);
	if(!parser) error("failed to create new parser");
	retptr = R_MakeExternalPtr( parser, redl_type_[ redl_parser ], R_NilValue );
	return(retptr);
}

void parse_into_redl_model(SEXP inparser, SEXP inuri, SEXP inmodel) { 
	librdf_parser* parser;
	librdf_uri* uri;
	librdf_model* model;
	CHECK_REDL_PARSER(inparser);
	parser = (librdf_parser *) R_ExternalPtrAddr( inparser );
	CHECK_REDL_URI(inuri);
	uri = (librdf_uri *) R_ExternalPtrAddr( inuri );
	CHECK_REDL_MODEL(inmodel);
	model = (librdf_model *) R_ExternalPtrAddr( inmodel );

	if(librdf_parser_parse_into_model(parser, uri, uri, model)) 
		error("Failed to parse RDF into model");
}

void free_redl_parser(SEXP inparser) {
	librdf_parser* parser;
	CHECK_REDL_PARSER(inparser);
	parser = (librdf_parser *) R_ExternalPtrAddr( inparser );
	librdf_free_parser(parser);
}

void print_redl_model(SEXP inmodel) {
	librdf_model* model;
	CHECK_REDL_MODEL(inmodel);
	model = (librdf_model *) R_ExternalPtrAddr( inmodel );
  	librdf_model_print(model, stdout);
}

/*
 PROTECT(ret = allocVector(STRSXP, 1));
                  SET_STRING_ELT(ret, 0, mkChar((char *) outs));
                  UNPROTECT(1);
                  return(ret);
*/


SEXP get_redl_model_size(SEXP inmodel) {
	librdf_model* model;
	int modsiz;
	SEXP ret;
	CHECK_REDL_MODEL(inmodel);
	model = (librdf_model *) R_ExternalPtrAddr( inmodel );
  	modsiz = librdf_model_size(model);
	PROTECT(ret = allocVector(INTSXP, 1));
	INTEGER(ret)[0] = modsiz;
	UNPROTECT(1);
	return(ret);
}

SEXP get_redl_storage_size(SEXP instorage) {
	librdf_storage* storage;
	int siz;
	SEXP ret;
	CHECK_REDL_STORAGE(instorage);
	storage = (librdf_storage *) R_ExternalPtrAddr( instorage );
  	siz = librdf_storage_size(storage);
	PROTECT(ret = allocVector(INTSXP, 1));
	INTEGER(ret)[0] = siz;
	UNPROTECT(1);
	return(ret);
}

SEXP get_redl_model_as_stream(SEXP inmodel) {
	librdf_model* model;
	librdf_stream* stream;
	SEXP retptr;
	CHECK_REDL_MODEL(inmodel);
	model = (librdf_model *) R_ExternalPtrAddr( inmodel );
	stream = librdf_model_as_stream( model );
	if(!stream) error("could not convert model to stream");
	retptr = R_MakeExternalPtr( stream, redl_type_[ redl_stream ], R_NilValue );
	return(retptr);
}

SEXP pop_statement_from_stream(SEXP instream) {
	librdf_stream* stream;
	librdf_statement* statement;
        SEXP retptr;
        CHECK_REDL_STREAM(instream);
	stream = (librdf_stream *) R_ExternalPtrAddr( instream );
	if(!stream) error("could not resolve stream pointer.");
        if(librdf_stream_end(stream)) return(R_NilValue);
	statement = librdf_stream_get_object(stream); /* start pop */
	if (!statement) error("null statement returned");
	librdf_stream_next(stream);  /* complete pop */
	retptr = R_MakeExternalPtr( statement, redl_type_[ redl_statement ], R_NilValue );
	return(retptr);
}

SEXP get_rdfnode_ref(SEXP instmt, SEXP which) {
	librdf_statement* statement;
	librdf_node* node;
        SEXP retptr;
        CHECK_REDL_STATEMENT(instmt);
	statement = (librdf_statement *) R_ExternalPtrAddr( instmt );
        if (INTEGER(which)[0] == IS_REDL_SUBJECT)
		node = librdf_statement_get_subject( statement );
        else if (INTEGER(which)[0] == IS_REDL_PREDICATE)
		node = librdf_statement_get_predicate( statement );
        else if (INTEGER(which)[0] == IS_REDL_OBJECT)
		node = librdf_statement_get_object( statement );
	if (!node) error("could not get node");
	retptr = R_MakeExternalPtr( node, redl_type_[ redl_node ], R_NilValue );
	return(retptr);
}

SEXP string_from_stmtref(SEXP instatement) {
	librdf_statement* statement;
	unsigned char * ser;
	SEXP ret;
        CHECK_REDL_STATEMENT(instatement);
	statement = (librdf_statement *) R_ExternalPtrAddr( instatement );
	ser = librdf_statement_to_string( statement );
 	PROTECT(ret = allocVector(STRSXP, 1));
        SET_STRING_ELT(ret, 0, mkChar((char *) ser));
        UNPROTECT(1);
        return(ret);
}
	
SEXP string_from_noderef(SEXP innode) {
	librdf_node* node;
	unsigned char * ser;
	SEXP ret;
        CHECK_REDL_NODE(innode);
	node = (librdf_node *) R_ExternalPtrAddr( innode );
	ser = librdf_node_to_string( node );
 	PROTECT(ret = allocVector(STRSXP, 1));
        SET_STRING_ELT(ret, 0, mkChar((char *) ser));
        UNPROTECT(1);
        return(ret);
}
	
SEXP new_redl_node_from_uri_string(SEXP inworld, SEXP string) {
	librdf_node* node;
	librdf_world* world;
	SEXP ret;
	CHECK_REDL_WORLD(inworld);
	world = (librdf_world *) R_ExternalPtrAddr( inworld );
	node = librdf_new_node_from_uri_string( world,
		CHAR(STRING_ELT(string,0)));
        ret= R_MakeExternalPtr( node, redl_type_[ redl_node ], R_NilValue );
        return(ret);
	}

==> Rredland_1.10.0.tar.gz: Rredland/src/Rredl.c <==
#include <stdio.h>
#include <redland.h>

#include <R.h>
#include <Rdefines.h>
#include <Rinternals.h>

/* note -- when adding a thing, increment the NUM_REDL_THINGS, and
   add a CHECK macro below */

#define NUM_REDL_THINGS 11
#define IS_REDL_SUBJECT 0
#define IS_REDL_PREDICATE 1
#define IS_REDL_OBJECT 2

enum redl_things {
	redl_world,
	redl_parser,
	redl_model,
	redl_storage,
	redl_stream,
	redl_subject,
	redl_predicate,
	redl_iterator,
	redl_statement,
	redl_node,
	redl_uri
};

static SEXP redl_type_[ NUM_REDL_THINGS ] ;

#define CHECK_REDL_REF(s,THING,CTYPE) do { \
	if (TYPEOF(s) != EXTPTRSXP || \
		R_ExternalPtrTag(s) != redl_type_[ THING ] ) \
		error("bad redland"  #CTYPE "reference"); \
	if ( (CTYPE *)R_ExternalPtrAddr(s) == (CTYPE *)NULL ) \
		error("CTYPE pointer is null"); \
	} while (0) ;

#define CHECK_REDL_WORLD(s) CHECK_REDL_REF(s, redl_world, librdf_world)
#define CHECK_REDL_PARSER(s) CHECK_REDL_REF(s, redl_parser, librdf_parser)
#define CHECK_REDL_MODEL(s) CHECK_REDL_REF(s, redl_model, librdf_model)
#define CHECK_REDL_STORAGE(s) CHECK_REDL_REF(s, redl_storage, librdf_storage)
#define CHECK_REDL_STREAM(s) CHECK_REDL_REF(s, redl_stream, librdf_stream)
#define CHECK_REDL_SUBJECT(s) CHECK_REDL_REF(s, redl_subject, librdf_subject)
#define CHECK_REDL_PREDICATE(s) CHECK_REDL_REF(s, redl_predicate, librdf_predicate)
#define CHECK_REDL_ITERATOR(s) CHECK_REDL_REF(s, redl_iterator, librdf_iterator)
#define CHECK_REDL_STATEMENT(s) CHECK_REDL_REF(s, redl_statement, librdf_statement)
#define CHECK_REDL_NODE(s) CHECK_REDL_REF(s, redl_node, librdf_node)
#define CHECK_REDL_URI(s) CHECK_REDL_REF(s, redl_uri, librdf_uri)
	
/**
 * Utility to allocate space for and copy CHAR(STRING_ELT(str_sxp,
 * elt_idx)) to char*. Allocation using R_alloc, so memory freed on
 * return to R.
 *
 */
char *char_R_alloc(const SEXP str_sxp, const int elt_idx) {
    SEXP elt = STRING_ELT(str_sxp, elt_idx);
    int len = length(elt);
    char *s = R_alloc(len+1, sizeof(char));
    strcpy(s, CHAR(elt));
    return s;
}
/*

openRedlWorld <- function() {
	.Call("get_open_redl_world", PACKAGE="Rredl" )
}
getRedlURI <- function(worldRef, uri) {
	if (!is(uri,"character") | length(uri)>1) stop("uri must be character string")
	.Call("set_redl_world_uri", worldRef, uri, PACKAGE="Rredl" )
}
getRedlStorageInCore <- function(worldRef) {
	.Call("get_storage_incore", worldRef, PACKAGE="Rredl" )
}
getRedlModel <- function(worldRef,storageRef) {
	.Call("get_redl_model", worldRef, storageRef, PACKAGE="Rredl" )
}
getRedlDefParser <- function(worldRef) {
	.Call("get_redl_default_parser", worldRef, PACKAGE="Rredl" )
}
 
*/
SEXP get_open_redl_world() {
	/* return an R external pointer to an open redl world */

	librdf_world* world;
	SEXP retptr;

	world=librdf_new_world();
	if (!world) error("could not create new world.");
	librdf_world_open(world);
	if (!world) error("could not open new world.");

	retptr = R_MakeExternalPtr( world, redl_type_[ redl_world ], R_NilValue );
	return(retptr);
}
SEXP free_redl_world(SEXP inworld) {
	librdf_world* world;
	CHECK_REDL_WORLD(inworld); 
	world = (librdf_world *) R_ExternalPtrAddr( inworld );
  	librdf_free_world(world);
	return(R_NilValue);
}
SEXP free_redl_storage(SEXP instorage) {
	librdf_storage* storage;
	CHECK_REDL_STORAGE(instorage);
	storage = (librdf_storage *) R_ExternalPtrAddr( instorage );
  	librdf_free_storage(storage);
	return(R_NilValue);
}
SEXP free_redl_model(SEXP inmodel) {
	librdf_model* model;
	CHECK_REDL_STORAGE(inmodel);
	model = (librdf_model *) R_ExternalPtrAddr( inmodel );
  	librdf_free_model(model);
	return(R_NilValue);
}
void get_and_free_redl_world() {
	librdf_world* world;
	world=librdf_new_world();
	librdf_world_open(world);
  	librdf_free_world(world);
}

SEXP get_redl_world_uri(SEXP inworld, SEXP inuri) {
	/* establish the source URI of RDF for a world */
	librdf_uri* uri;
	librdf_world* world;
	const unsigned char* uristring;
	SEXP retptr;

	CHECK_REDL_WORLD(inworld);
	world = (librdf_world *) R_ExternalPtrAddr( inworld );

	uristring = (const unsigned char*) CHAR(STRING_ELT(inuri, 0));
	uri = librdf_new_uri( world, uristring );
	if(!uri) error("Failed to create URI");
	retptr = R_MakeExternalPtr( uri, redl_type_[ redl_uri ], R_NilValue );
	return(retptr);
}

SEXP new_storage_incore(SEXP inworld) {
	librdf_storage* storage;
	librdf_world* world;
	SEXP retptr;

	CHECK_REDL_WORLD(inworld);
	world = (librdf_world *) R_ExternalPtrAddr( inworld );
  	storage=librdf_new_storage(world, "memory", "test", NULL);
	if(storage == NULL) error("failed to create new storage");
	retptr = R_MakeExternalPtr( storage, redl_type_[ redl_storage ], R_NilValue );
	return(retptr);
}

SEXP call_new_storage(SEXP inworld, SEXP type,
		SEXP name, SEXP rshash) {
	librdf_storage* storage;
	librdf_world* world;
	SEXP retptr;
	CHECK_REDL_WORLD(inworld);
	world = (librdf_world *) R_ExternalPtrAddr( inworld );
	if (strlen(CHAR(STRING_ELT(rshash,0))) == 0)
		storage=librdf_new_storage(world, char_R_alloc(type, 0),
			char_R_alloc(name, 0), NULL);
	else
		storage=librdf_new_storage(world, char_R_alloc(type, 0),
			char_R_alloc(name, 0),
			char_R_alloc(rshash, 0));
	if(storage == NULL) error("failed to create new storage");
	retptr = R_MakeExternalPtr( storage, redl_type_[ redl_storage ], R_NilValue );
	return(retptr);
}

SEXP new_storage_bdb(SEXP inworld, SEXP stoname) {
	librdf_storage* storage;
	librdf_world* world;
	SEXP retptr;

	CHECK_REDL_WORLD(inworld);
	world = (librdf_world *) R_ExternalPtrAddr( inworld );
	storage=librdf_new_storage(world, "hashes", char_R_alloc(stoname, 0), "hash-type='bdb',dir='.'");
	if(storage == NULL) error("failed to create new storage");
	retptr = R_MakeExternalPtr( storage, redl_type_[ redl_storage ], R_NilValue );
	return(retptr);
}

SEXP revive_storage_testbdb(SEXP inworld, SEXP stoname) {
	librdf_storage* storage;
	librdf_world* world;
	SEXP retptr;

	CHECK_REDL_WORLD(inworld);
	world = (librdf_world *) R_ExternalPtrAddr( inworld );
	storage=librdf_new_storage(world, "hashes", char_R_alloc(stoname, 0), "hash-type='bdb',dir='.'");
	if(storage == NULL) error("failed to revive existing storage");
	retptr = R_MakeExternalPtr( storage, redl_type_[ redl_storage ], R_NilValue );
	return(retptr);
}

SEXP get_redl_model(SEXP inworld, SEXP instorage) {
	librdf_storage* storage;
	librdf_world* world;
	librdf_model* model;
	SEXP retptr;

	CHECK_REDL_WORLD(inworld);
	world = (librdf_world *) R_ExternalPtrAddr( inworld );
	CHECK_REDL_STORAGE(instorage);
	storage = (librdf_storage *) R_ExternalPtrAddr( instorage );

  	model=librdf_new_model(world, storage, NULL);
	if(!model) error("failed to create new model");
	retptr = R_MakeExternalPtr( model, redl_type_[ redl_model ], R_NilValue );
	return(retptr);
}

SEXP get_redl_default_parser(SEXP inworld) {
	librdf_parser* parser;
	librdf_world* world;
	SEXP retptr;

	CHECK_REDL_WORLD(inworld);
	world = (librdf_world *) R_ExternalPtrAddr( inworld );

	parser=librdf_new_parser(world, NULL, NULL, NULL);
	if(!parser) error("failed to create new parser");
	retptr = R_MakeExternalPtr( parser, redl_type_[ redl_parser ], R_NilValue );
	return(retptr);
}

void parse_into_redl_model(SEXP inparser, SEXP inuri, SEXP inmodel) { 
	librdf_parser* parser;
	librdf_uri* uri;
	librdf_model* model;
	CHECK_REDL_PARSER(inparser);
	parser = (librdf_parser *) R_ExternalPtrAddr( inparser );
	CHECK_REDL_URI(inuri);
	uri = (librdf_uri *) R_ExternalPtrAddr( inuri );
	CHECK_REDL_MODEL(inmodel);
	model = (librdf_model *) R_ExternalPtrAddr( inmodel );

	if(librdf_parser_parse_into_model(parser, uri, uri, model)) 
		error("Failed to parse RDF into model");
}

void free_redl_parser(SEXP inparser) {
	librdf_parser* parser;
	CHECK_REDL_PARSER(inparser);
	parser = (librdf_parser *) R_ExternalPtrAddr( inparser );
	librdf_free_parser(parser);
}

void print_redl_model(SEXP inmodel) {
	librdf_model* model;
	CHECK_REDL_MODEL(inmodel);
	model = (librdf_model *) R_ExternalPtrAddr( inmodel );
  	librdf_model_print(model, stdout);
}

/*
 PROTECT(ret = allocVector(STRSXP, 1));
                  SET_STRING_ELT(ret, 0, mkChar((char *) outs));
                  UNPROTECT(1);
                  return(ret);
*/


SEXP get_redl_model_size(SEXP inmodel) {
	librdf_model* model;
	int modsiz;
	SEXP ret;
	CHECK_REDL_MODEL(inmodel);
	model = (librdf_model *) R_ExternalPtrAddr( inmodel );
  	modsiz = librdf_model_size(model);
	PROTECT(ret = allocVector(INTSXP, 1));
	INTEGER(ret)[0] = modsiz;
	UNPROTECT(1);
	return(ret);
}

SEXP get_redl_storage_size(SEXP instorage) {
	librdf_storage* storage;
	int siz;
	SEXP ret;
	CHECK_REDL_STORAGE(instorage);
	storage = (librdf_storage *) R_ExternalPtrAddr( instorage );
  	siz = librdf_storage_size(storage);
	PROTECT(ret = allocVector(INTSXP, 1));
	INTEGER(ret)[0] = siz;
	UNPROTECT(1);
	return(ret);
}

SEXP get_redl_model_as_stream(SEXP inmodel) {
	librdf_model* model;
	librdf_stream* stream;
	SEXP retptr;
	CHECK_REDL_MODEL(inmodel);
	model = (librdf_model *) R_ExternalPtrAddr( inmodel );
	stream = librdf_model_as_stream( model );
	if(!stream) error("could not convert model to stream");
	retptr = R_MakeExternalPtr( stream, redl_type_[ redl_stream ], R_NilValue );
	return(retptr);
}

SEXP pop_statement_from_stream(SEXP instream) {
	librdf_stream* stream;
	librdf_statement* statement;
        SEXP retptr;
        CHECK_REDL_STREAM(instream);
	stream = (librdf_stream *) R_ExternalPtrAddr( instream );
	if(!stream) error("could not resolve stream pointer.");
        if(librdf_stream_end(stream)) return(R_NilValue);
	statement = librdf_stream_get_object(stream); /* start pop */
	if (!statement) error("null statement returned");
	librdf_stream_next(stream);  /* complete pop */
	retptr = R_MakeExternalPtr( statement, redl_type_[ redl_statement ], R_NilValue );
	return(retptr);
}

SEXP get_rdfnode_ref(SEXP instmt, SEXP which) {
	librdf_statement* statement;
	librdf_node* node;
        SEXP retptr;
        CHECK_REDL_STATEMENT(instmt);
	statement = (librdf_statement *) R_ExternalPtrAddr( instmt );
        if (INTEGER(which)[0] == IS_REDL_SUBJECT)
		node = librdf_statement_get_subject( statement );
        else if (INTEGER(which)[0] == IS_REDL_PREDICATE)
		node = librdf_statement_get_predicate( statement );
        else if (INTEGER(which)[0] == IS_REDL_OBJECT)
		node = librdf_statement_get_object( statement );
	if (!node) error("could not get node");
	retptr = R_MakeExternalPtr( node, redl_type_[ redl_node ], R_NilValue );
	return(retptr);
}

SEXP string_from_stmtref(SEXP instatement) {
	librdf_statement* statement;
	unsigned char * ser;
	SEXP ret;
        CHECK_REDL_STATEMENT(instatement);
	statement = (librdf_statement *) R_ExternalPtrAddr( instatement );
	ser = librdf_statement_to_string( statement );
 	PROTECT(ret = allocVector(STRSXP, 1));
        SET_STRING_ELT(ret, 0, mkChar((char *) ser));
        UNPROTECT(1);
        return(ret);
}
	
SEXP string_from_noderef(SEXP innode) {
	librdf_node* node;
	unsigned char * ser;
	SEXP ret;
        CHECK_REDL_NODE(innode);
	node = (librdf_node *) R_ExternalPtrAddr( innode );
	ser = librdf_node_to_string( node );
 	PROTECT(ret = allocVector(STRSXP, 1));
        SET_STRING_ELT(ret, 0, mkChar((char *) ser));
        UNPROTECT(1);
        return(ret);
}
	
SEXP new_redl_node_from_uri_string(SEXP inworld, SEXP string) {
	librdf_node* node;
	librdf_world* world;
	SEXP ret;
	CHECK_REDL_WORLD(inworld);
	world = (librdf_world *) R_ExternalPtrAddr( inworld );
	node = librdf_new_node_from_uri_string( world,
		(const unsigned char*) char_R_alloc(string,0));
        ret= R_MakeExternalPtr( node, redl_type_[ redl_node ], R_NilValue );
        return(ret);
	}
